import utils
from os import environ
from subprocess import call
from termcolor import colored
import fileinput

# list all items
def ls(path, args):
    with open(path) as todo:
        # ls acts as a refresh, so current todo file is reorganized before printing
        utils.reorganize_todo(path)
        for item in todo:
            i = utils.unpack(item)
            print colored(i['num'],'red') + '.' + colored(i['text'], 'white')


# add new item
def a(path, args):
    if len(args) == 1:
        # no specific number is given, append the item to end
        with open(path, 'a') as todo:
            text = args[0]
            todo.write(''.join([text,'\n']))

    elif len(args) == 2 and int(args[0]):
        # insert the item as the given numbered todo
        number, text = args
        index = int(number) - 1
        def insert_item(todo_file):
            contents = todo_file.readlines()
            contents.insert(index, ''.join([text, '\n']))
            return contents
        utils.read_then_write(path, insert_item)

    else:
        print 'not applicable to given arguments'

# edit the current todo file
def e(path, args):
    EDITOR = environ.get('EDITOR', 'vim')
    call([EDITOR, path])

# delete an item
def d(path, args):
    number = int(args[0])
    if number:
        def delete_item(todo_file):
            contents = todo_file.readlines()
            contents = [l for l in contents if utils.unpack(l)['num'] != number]
            return contents
        utils.read_then_write(path, delete_item)



from time import strftime
from os import path, listdir
from shutil import copyfile

def get_todo_path(directory):
    today = strftime('%Y%m%d')
    todo_path = directory + '/'+ today + '.md'

    # if today's todo.md does not exist, create the file
    if not path.isfile(todo_path):
        previous_todo_file = check_previous_todo(directory)

        if not previous_todo_file:
            # create an empty file
            open(todo_path, 'w')
        else:
            # copy previous days todos to todays todo.md
            copyfile(directory + '/' + previous_todo_file, todo_path)

    return todo_path

def check_previous_todo(directory):
    # get all todo.md's filtering all hidden files
    todos = [t for t in listdir(directory) if t[0] != '.']
    todos.sort()
    if(len(todos)):
        return todos[-1]
    else:
        return False

def reorganize_todo(file_path):
    with open(file_path, 'r+') as unorganized:
        organized = []
        current_line_number = 1
        for line in unorganized.readlines():
            # TODO: check is below is a nice way of doing this
            line_list = line.split('.',1)
            if len(line_list) == 2 and line_list[0].isdigit():
                [number,text] = line_list
                text = text.lstrip()
            else:
                text = ''.join(line_list)

            organized.append(''.join([str(current_line_number), '. ', text]))

            current_line_number = current_line_number + 1

        unorganized.seek(0)
        unorganized.writelines(organized)

def unpack(item):
    item_list = item.rstrip().split('.',1)
    res = {}
    res['num'] = int(item_list[0])
    res['text'] = item_list[1]
    return res

def read_then_write(path, fn_to_read):
    with open(path, 'r') as f:
        new_contents = fn_to_read(f)
        with open(path, 'w') as f_write:
            f_write.writelines(new_contents)




from distutils.core import setup

setup(
    name = 'todo-md',
    version = '0.0.2',
    author = 'onur satici',
    author_email = 'onursatici@gmail.com',
    packages = ['todo_md'],
    install_requires = [
        'termcolor'
    ],
    scripts = ['todo'],
    description = 'txt file backed todo manager',
    long_description = open('README.txt').read()
)
